import markdown

from django import template
from django.core.cache import cache

from django_tiniest_cms.models import Content


def content(parser, token):
    try:
        tag_name, name = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires an argument" % token.contents.split()[0]
        )
    if not (name[0] == name[-1] and name[0] in ('"', "'")):
        raise template.TemplateSyntaxError(
            "%r tag's name argument should be in quotes" % tag_name
        )
    name = name.strip('"\'')
    nodelist = parser.parse(('endcontent',))
    parser.delete_first_token()
    return ContentNode(nodelist, content_name=name)


class ContentNode(template.Node):
    def __init__(self, nodelist, content_name):
        self.nodelist = nodelist
        self.content_name = content_name

    def render(self, context):
        try:
            key = 'tiniest-content-{0}'.format(self.content_name)
            cached_content = cache.get(key)
            if cached_content is not None:
                content_html = cached_content
            else:
                content = Content.objects.get(name=self.content_name)
                content_html = markdown.markdown(
                    content.content,
                    output_format='html5',
                    safe_mode='escape'
                )
                cache.set(key, content_html, 3600)
            return content_html
        except Content.DoesNotExist:
            return self.nodelist.render(context)


register = template.Library()
register.tag('content', content)



from django.conf import settings
from django.core.cache import cache
from django.db import IntegrityError, connection, reset_queries
from django.template import Context, Template
from django.test import TestCase

from django_tiniest_cms.models import Content

TEMPLATE = Template("""{% load tiniest_cms %}
{% content 'root' %}
default stuff
{% endcontent %}
""")

CONTEXT = Context(dict())

class ContentTestCase(TestCase):
    def setUp(self):
        settings.DEBUG = True
        cache.clear()

    def test_content_names_are_unique(self):
        Content.objects.create(name='root', content='different')
        with self.assertRaises(IntegrityError):
            Content.objects.create(name='root', content='different')

    def test_no_content(self):
        self.assertIn('default stuff', TEMPLATE.render(CONTEXT))

    def test_with_content(self):
        Content.objects.create(name='root', content='different')
        reset_queries()
        self.assertIn('different', TEMPLATE.render(CONTEXT))
        self.assertEqual(len(connection.queries), 1)

    def test_content_rendered_as_markdown(self):
        Content.objects.create(
            name='root',
            content='*embiggen* and **cromulent**'
        )
        self.assertIn('<em>embiggen</em>', TEMPLATE.render(CONTEXT))

    def test_html_content_escaped(self):
        Content.objects.create(name='root', content='<script>CSRF</script>')
        self.assertNotIn('<script>CSRF</script>', TEMPLATE.render(CONTEXT))

    def test_content_cached(self):
        Content.objects.create(name='root', content='different')
        reset_queries()
        TEMPLATE.render(CONTEXT) # first rendering
        TEMPLATE.render(CONTEXT) # second rendering
        self.assertEqual(len(connection.queries), 1)

    def test_content_cache_updated_on_save(self):
        content = Content.objects.create(name='root', content='different')
        self.assertIn('different', TEMPLATE.render(CONTEXT))

        content.content = 'other'
        content.save()

        self.assertIn('other', TEMPLATE.render(CONTEXT))



from django.db import models
from django.core.cache import cache

class Content(models.Model):
    name = models.CharField(max_length=512, unique=True)
    content = models.TextField()

    def save(self, *args, **kwargs):
        key = 'tiniest-content-{0}'.format(self.name)
        cache.delete(key)
        return super(Content, self).save(*args, **kwargs)



from setuptools import setup

setup(
    name='django-tiniest-cms',
    version='0.9.1',
    description='django-tiniest-cms is a seriously minimalist Django CMS',
    long_description=open('README').read(),
    author='Gautier Hayoun',
    author_email='ghayoun@gmail.com',
    url='https://github.com/Gautier/django-tiniest-cms',
    license='MIT',
    packages=['django_tiniest_cms', 'django_tiniest_cms.templatetags'],
    install_requires=['Markdown==2.4.1']
)
